!FILE       :	2deuns.f90
!UNIT NAME  :	two_deuns	(PROGRAM)
!DESCRIPTION:	This is the main program, from here all calling is done.
!		Also the runge kutta integration is done in this level as well as data
!		output and convergence check
!		The allocatable arrays are initialized here within the array_allocation subroutine
!		Written by Joel GUERRERO
!		REV01-01/2007, REV02-05/2007, REV03-05/2010, REV04-09/2020
!		Email: joegi.geo@yahoo.com

program two_deuns


use shared_data

implicit none





call cpu_time(time_begin)



!*********************************************************************************
!Loading the global mesh data structure
call read_mesh
!*********************************************************************************



!*********************************************************************************
!Some array allocation outside loop 
call array_allocation
!*********************************************************************************



!*********************************************************************************
!Global variables declaration
call var_init
!*********************************************************************************



!*********************************************************************************
!Reading the application dependent parameters
call read_data
!*********************************************************************************



!*********************************************************************************
!Set the initial guess
call init_guess
!*********************************************************************************



!*********************************************************************************
!Computation of the control volume and triangle areas
call ele_area
!*********************************************************************************



!*********************************************************************************
!Computation of control volume lenght
call ele_lenght
!*********************************************************************************



!*********************************************************************************
!Reconstruction of the mesh edges 
call edge_recons
!*********************************************************************************



!*********************************************************************************
!Initializing the physical solution & swapping the initial physical solution & computation of the initial pressures
call sol_init
!*********************************************************************************



!*********************************************************************************
!Writes data dumped to screen in solution_control.txt file
if(restart.eq.0) then
	open(95,file='solution_control.txt',status='unknown')
else
	open(95,file='solution_control.txt',status='unknown',position='append')
	write(95,*)
	write(95,*)'*********************************************'
	write(95,*)'*********************************************'
	write(95,*)'*********************************************'
	write(95,*)'    CONTINUATION FROM A PREVIOUS SOLUTION    '
	write(95,*)'*********************************************'
	write(95,*)'*********************************************'
	write(95,*)'*********************************************'
	write(95,*)
endif

write(95,*)'*********************************************'
write(95,*)
write(95,*)'   Actual iteration =  ',iter_actual
write(95,*)'   Actual residual  =  ',sum
!*********************************************************************************



!*********************************************************************************
!Some array allocation outside loop 
call screen_control
!*********************************************************************************



!*********************************************************************************
!Saving the initial physical solution and mesh
call initial_output
!*********************************************************************************



!*********************************************************************************
!Writes residuals and some other information to RESIDUAL file
open(30,file="residual.txt",status="unknown")
rewind(30)
!*********************************************************************************



!*********************************************************************************
!****************************** START TIME LOOP **********************************
!*********************************************************************************
!Physical time loop
!
iteration_loop:do	i=1,iter_max


!USE THIS ONLY IF YOU WANT TO TIME THE CURRENT ITER
call cpu_time(time_begin_iter)					



	iter_actual = iter_actual + 1


	!******************************************************************************
	!Delta t computation in base of cell lenght and cfl number
	call cell_cfl
	!******************************************************************************


	!******************************************************************************
	!START OF RUNGE KUTTA TIME INTEGRATION
	!
	runge_kutta_loop:do ialpha=1,irk


		!**************************************************************************
		!Call explicit scheme formulation here
		call explicit_scheme
		!**************************************************************************


		!**************************************************************************
		!Pressures update after explicit scheme
        	call pres_update
		!**************************************************************************


		!**************************************************************************
		!Computation of residual
		if (ialpha.eq.1) then

			sum      = 0.0

			do is=1,n_ver
				sum  = sum + convar_exchange(1,is)*convar_exchange(1,is)+ convar_exchange(2,is)*convar_exchange(2,is)+ &
				convar_exchange(3,is)*convar_exchange(3,is)+ convar_exchange(4,is)*convar_exchange(4,is)
			enddo

			!sum      = sqrt(sum)/(float(n_ver))		!I USED FLOAT IN ORDER TO CONVERT n_ver TO REAL
			sum      = sqrt(sum)/n_ver

			if ((iter_actual - iter_prev) .eq. 1) then 	
				sum_prev = sum
			endif

			write(30,*) iter_actual,sum/sum_prev,sum

		endif   
		!End of Computation of residual
		!************************************************************************** 
		              

	enddo runge_kutta_loop
	!
	!END OF RUNGE KUTTA TIME INTEGRATION
	!******************************************************************************



	!******************************************************************************
	!Swapping of the new physical solution the new variable is to be use within the RK loop
	do is=1,n_ver	
		do ivar=1,num_var
         
			vars_prev(ivar,is)  = vars_actual(ivar,is)

		enddo	
	enddo
	!******************************************************************************



	!******************************************************************************
	!Check convergence, stopping parameters and dump some data to the screen
	call convergence_check

	if(stop_program.eq.1) then
		exit iteration_loop
	endif
	!******************************************************************************



!USE THIS ONLY IF YOU WANT TO TIME THE CURRENT ITER
call cpu_time(time_end_iter)					
print *,'Total time for iteration ',iter_actual,' = ',time_end_iter - time_begin_iter,' seconds'		

	

enddo iteration_loop
!*********************************************************************************
!*************************** END TIME LOOP ***************************************
!*********************************************************************************



call tecplot_last_output

call vtk_output


call cpu_time(time_end)
print *,'Total time for execution = ',time_end - time_begin,' seconds'
write(95,*)'Total time for execution = ',time_end - time_begin,' seconds'


print *	
print *,'END OF NORMAL EXECUTION'
write(95,*)
write(95,*)'END OF NORMAL EXECUTION'



close(30)
close(95)





end program two_deuns
