!FILE       :	read_mesh.f90
!UNIT NAME  :	read_mesh	(SUBROUTINE)
!DESCRIPTION:	This subroutine reads the initial mesh from MESH file
!		It also identifies all the bounary vertices
!		Written by Joel GUERRERO on 
!		REV01-01/2007, REV02-09/2020
!		Email: joegi.geo@yahoo.com

subroutine read_mesh


use shared_data

implicit none





!****************************************************************************************
!LOCAL VARIABLES
!
real::xmin			=0
real::xmax			=0
real::ymin			=0
real::ymax			=0
!
!ATTENTION: ALL DUMMY VARIABLES DECLARED AS LOCAL VARIABLES ARE TEMPORAL
!THEY ARE ONLY USED WITHIN THE SUBROUTINE
!****************************************************************************************



num_internal_ver		=0
num_slip_ver			=0
num_noslip_ver			=0
num_outlet_ver			=0
num_inlet_ver			=0
num_inpro_ver			=0
num_total_ver			=0
num_total_boundary_ver		=0



!****************************************************************************************
!OPEN FILE
!
!open(30,file="MESH",status="old")
!open(30,file="output.txt",status="old")
open(30,file="mesh.txt",status="old")
rewind(30)

read(30, * ) n_ver,n_ele
print *,'*********************************************'
print *
print *,'NUMBER OF VERTICES       : ', n_ver
print *,'NUMBER OF TRIANGLES      : ', n_ele
print *
print *,'*********************************************'



!****************************************************************************************
!ARRAY ALLOCATION & INITIALIZATION
!
allocate(coor(2,n_ver))							!vertex coordinates array
allocate(connectivity(3,n_ele))						!connectivity array
allocate(array_boundary_ver(n_ver))					!array of boundary flags
allocate(array_slip_ver(n_ver))						!array of slipping ver
allocate(array_noslip_ver(n_ver))					!array of noslipping ver
allocate(array_outlet_ver(n_ver))					!array of downstream ve
allocate(array_inlet_ver(n_ver))					!array of upstream ver
allocate(array_inpro_ver(n_ver))					!array of inflow profile ver

coor						=0
connectivity					=0
array_boundary_ver				=0
array_slip_ver					=0
array_noslip_ver				=0
array_outlet_ver				=0
array_inlet_ver					=0
array_inpro_ver					=0

max_ele=n_ele								!MAXIMUM NUMBER OF ELEMENTS
max_edges=n_ver+n_ele							!MAXIMUM NUMBER OF EDGES
!****************************************************************************************



!****************************************************************************************
!READ INPUT FILE
!
do i=1,n_ver 
 
	read(30,*) index,coor(1,i),coor(2,i),array_boundary_ver(i)

enddo

do j=1,n_ele

	read(30,*) k,connectivity(1,j),connectivity(2,j),connectivity(3,j),i

enddo

close(30)
!****************************************************************************************



!****************************************************************************************
!FIND MIN AND MAX
!

xmin         =  1.0e+4
xmax         = -1.0e+4
ymin         =  1.0e+4
ymax         = -1.0e+4

      	do is=1,n_ver
        	xmin      = min(xmin , coor(1,is))
         	xmax      = max(xmax , coor(1,is))
         	ymin      = min(ymin , coor(2,is))
        	ymax      = max(ymax , coor(2,is))
	enddo

	print *  
      	print *,'*********************************************'
      	print *,'DOMAIN MAXIMUM AND MINIMUM   :'
      	print *,'xmin = ', xmin
      	print *,'xmax = ', xmax
      	print *,'ymin = ', ymin
      	print *,'ymax = ', ymax
	print *,'*********************************************'
!****************************************************************************************



!****************************************************************************************
!VERTICES RENUMERATION
!
      num_internal_ver			= 0
      num_slip_ver			= 0
      num_noslip_ver			= 0
      num_outlet_ver			= 0
      num_inlet_ver			= 0
      num_inpro_ver			= 0


      do is=1,n_ver

		if (array_boundary_ver(is) .eq. 0) then
		
			num_internal_ver  = num_internal_ver+1

		endif


		if(array_boundary_ver(is).eq.2.or.(array_boundary_ver(is).eq.3.and.vis_type.eq.0)) then

			num_slip_ver  = num_slip_ver+1
			array_slip_ver(num_slip_ver)=is

		endif


!		numc(is)=0
!		if (array_boundary_ver(is).eq.3) then

!			num_noslip_ver = num_noslip_ver+1 
!			log3(num_noslip_ver)=is
!			numc(is)=num_noslip_ver
!
!		endif


		if (array_boundary_ver(is) .eq. 4) then

			num_outlet_ver  = num_outlet_ver+1
			array_outlet_ver(num_outlet_ver)=is

		endif

		if (array_boundary_ver(is) .eq. 5) then

			num_inlet_ver  = num_inlet_ver+1
			array_inlet_ver(num_inlet_ver)=is

		endif

		if (array_boundary_ver(is) .eq. 6) then

			num_inpro_ver =num_inpro_ver+1
			array_inpro_ver(num_inpro_ver)=is

		endif

	enddo
!****************************************************************************************



!****************************************************************************************
!ARRAY DEALLOCATION IF THEY ARE NOT USED


if(num_slip_ver.eq.0) then
	
	deallocate(array_slip_ver)

!else

!	allocate(array_temp(num_slip_ver))
!	array_temp=array_slip_ver
!	deallocate(array_slip_ver)
!	allocate(array_slip_ver(num_slip_ver))
!	array_slip_ver=0
!	array_slip_ver=array_temp
!	deallocate(array_temp)
!	write(*,*) array_slip_ver

endif

if(num_noslip_ver.eq.0) then

	deallocate(array_noslip_ver)

endif

if(num_inpro_ver.eq.0) then

	deallocate(array_inpro_ver)

endif
!****************************************************************************************



!****************************************************************************************
!DUMP INFORMATION TO SCREEN
!
num_total_boundary_ver		=	num_slip_ver+num_noslip_ver+num_outlet_ver+num_inlet_ver+num_inpro_ver
num_total_ver			=	num_internal_ver+num_slip_ver+num_noslip_ver+num_outlet_ver+num_inlet_ver+num_inpro_ver

print *  
print *,'*********************************************'
print *,'Internal mesh vertices   : ',num_internal_ver
print *,'Slipping vertices        : ',num_slip_ver
print *,'No-slipping vertices     : ',num_noslip_ver
print *,'Upstream   vertices      : ',num_inlet_ver
print *,'Downstream vertices      : ',num_outlet_ver
print *,'Inflow profile vertices  : ',num_inpro_ver
print *,'Total for verification   : ',num_total_ver
print *,'*********************************************'
print *  
!****************************************************************************************          



!****************************************************************************************


end subroutine read_mesh
